# APRIORI_PYTHON_HEADER_START#################################################
#                                                          vi: set ts=8: sw=4:
# @(#) $Id: HtmlArea.py,v 1.2 2003/07/25 14:58:18 th Exp $
#
#     Copyright (C)2001 APRIORI business solutions. All Rights Reserved.
#
#         This software is free for non-commercial use, and for use
#                   with APRIORI Editorial System (AES).
#                        Use it on your own risk!
#
# $Log: HtmlArea.py,v $
# Revision 1.2  2003/07/25 14:58:18  th
# should work!
#
#
# APRIORI_PYTHON_HEADER_END###################################################
import ExtensionClass
from Globals import DTMLFile

# map the old button keys (HtmlArea 2.03) to the new ones
# for backward compatibility
oldButtonsMap = { # v2.03 name,      v3 name
                  'OrderedList':    'insertorderedlist',
                  'UnOrderedList':  'insertunorderedlist',
                  'Outdent':        'outdent',
                  'Indent':         'indent',
                  'fontstyle':      'formatblock',
                  'HorizontalRule': 'inserthorizontalrule',
                  'Createlink':     'createlink',
                  'InsertImage':    'insertimage',
                  'InsertTable':    'inserttable',
                  'HtmlMode':       'htmlmode',
                  'backcolor':      'hilitecolor',
                  'help':           'showhelp',
                 }

class HtmlAreaMixin(ExtensionClass.Base):
    """
    A mixin class just to provide the DTML method HtmlArea_script.
    It doesn't define an own object type you could create objects of.

    You don't *really* need to use the mixin class (although this
    is recommended); you may as well simply insert the contents of
    the method into the method[s] you intend to use.
    
    Methods:

    **htmlArea_prefix** -- includes the editor.js if the browser
                           is recognised as a supported one
                           (MSIE/Win >= 5.5, or Mozilla >= 1.3).
                           You may specify the language for the button
                           tooltips, e.g. lang='de'.

    **htmlArea_editor** -- creates an editor for the specified textarea,
                           several options

    **htmlArea_style** -- creates a html link to include the standard
                          stylesheet
    """
    htmlArea_prefix = DTMLFile('dtml/htmlArea_prefix', globals(),
                               lang='en') # this default might change
                                          # to use the first language of
                                          # the user's preferred which we support

    def htmlArea_editor(self, 
                        name, # name of the textarea
                        toolbar='standard',
                        width=None,
                        height=None,
                        **buttons):
        """
        Inserts the code to turn the textarea with the given *name*
        into a WYSIWYG editor. Must be called in the html body;
        requires the use of --> htmlArea_prefix in the html head.

        Arguments:

        **name** -- the name of the textarea

        **height** -- a height specification for the editor (px or percentage);
                      default: '200px' (may change).

                      Note:

                          when neither *height* nor *width* nor any toolbar
                          modification is specified, just the standard
                          editor is created, without any *config* specification!

        **width** -- a width specification for the editor (px or percentage)
                      default: '100%' (may change)

        **toolbar** -- allows the preselection of a set of available buttons.
                     Currently supported (string values):

                     *simple* -- bold, italic, underline, about only

                     *restricted* -- adds justify buttons and links

                     *normal* -- adds lists and indentations (includes *restricted*)

                     *standard* -- what htmlArea 3 considers standard
                       (doesn't explicitely define the toolbar unless 
                       specific buttons requested, see below)

                     *all* -- all standard buttons (of htmlarea 2.03) known to
                              this Zope product
                     
        **\*\*buttons** -- allows to switch on or off single buttons
                     by specifying named arguments. Any htmlArea 2 button keys
                     are translated to the new names
        """
        if not (height or width or (toolbar != 'standard') or buttons):
            return """\n<script language="javascript1.2"><!--
HTMLArea.replace('%s');
//--></script>""" % name

        # defer: empty attribute, prevents the script from generating code; newer MSIEs only
        res = """\n<script language="javascript1.2" defer><!--\n"""
        
        tint = type(0)
        if height is None:
            height = '200px'
        elif type(height) == tint:
            height = '%dpx' % height
        
        if width is None:
            width = '100%'
        elif type(width) == tint:
            if width > 100:
                width = '%dpx' % width
            else:
                width = '%d%%' % width
        # DEBUG
        if 0:
            height = 100
            width= 200

        if toolbar == 'all':
            _toolbar = { 'showhelp': 0, # not yet filled with useful content in htmlArea 3 alpha
                         # 'formatblock': 0, # fontstyle didn't work in 2.03
                        }
            default = 1
        else:
            if not (toolbar in ('simple', 'restricted', 'normal', 'standard')):
                res += '// unknown toolbar set: "%s";\n' % toolbar
                toolbar = buttons and 'normal' or 'standard'
                res += '// changed to: "%s"\n' % toolbar
            elif toolbar == 'standard' and buttons:
                toolbar = 'normal'

            default = 0
            _toolbar = { 'bold': 1,
                         'italic': 1,
                         'underline': 1,
                         'about': 1,
                        }
            if toolbar in ('restricted', 'normal'):
                _toolbar.update({'justifyleft': 1,
                                 'justifyright': 1,
                                 'justifycenter': 1,
                                 'justifyfull': 1,
                                })
                if toolbar in ('normal',):
                    for k in ['fontname', 'fontsize', 'formatblock',
                              'strikethrough', 'subscript', 'superscript',
                            'insertorderedlist','insertunorderedlist','outdent','indent',
                            'forecolor','hilitecolor', 'textindicator',
                            'inserthorizontalrule','createlink','insertimage','inserttable','htmlmode',
                            'popupeditor']:
                        _toolbar[k] = 1
        if buttons:
            for k in buttons.keys():
                tmp = oldButtonsMap.get(k)
                if tmp is not None:
                    # don't override a new key with an old one:
                    if buttons.get(tmp) is None:
                        _toolbar[tmp] = buttons[k]
                else:
                    _toolbar[k] = buttons[k]
            # it would be that simple if there weren't the old keys: :-/
            # _toolbar.update(buttons)

        res += 'var config = new HTMLArea.Config();\n'
        if width:
            res += "config.width = '%s';\n" % width
        if height:
            res += "config.height = '%s';\n" % height
        lines = []

        if buttons or (toolbar != 'standard'):
            tmpline = []
            for keyset in ( ('fontname', 'fontsize', 'formatblock',
                             'bold', 'italic', 'underline',
                             'strikethrough', 'subscript', 'superscript',),
                            ('justifyleft', 'justifycenter', 'justifyright', 
                             'justifyfull'), # new in htmlArea 3
                            ('insertorderedlist', 'insertunorderedlist', 'outdent', 'indent',),
                            ('forecolor', 'hilitecolor', 'textindicator',),
                            ('inserthorizontalrule', 'createlink', 'insertimage', 'inserttable'),
                            ('htmlmode', 'about', 'showhelp'),
                           ):
                for key in keyset:
                    if _toolbar.get(key, default):
                        tmpline.append(key)
                if tmpline:
                    lines.append("'%s'" % "', '".join(tmpline))
                    tmpline = []
            
        if lines:
            res += 'config.toolbar = [[%s]];\n' % \
                   ", 'separator'],\n                  [".join(lines)
                         
        res += "HTMLArea.replace('%s', config);\n" % name
        res += '//--></script>'
        return res

    def htmlArea_style(self):
        """
        simply includes the standard styles
        """
        return '<link rel="stylesheet" type="text/css" href="/misc_/HtmlArea3/htmlarea.css">'
    
        # this doesn't work, though recommended by interactive tools:
        return '<style type="text/css">@import url(/misc_/HtmlArea3/htmlarea.css)</style>'
